% ----------------------------------------------------------------
%   DDSGT.M   [Simulate] button DDS callback.
%   Main simulation executor. This file can also by run
%   from the MATLAB command line, not necessarily from GUI.
%   In that case gShell variable must be set to 0.
%
%   Input:      DDS input variables
%                 iN .. phase accumulator width (bits)
%                 iP .. number of MSB phase accumulator bits
%                       used for phase/amplitude conversion
%                 iF .. tuning number
%                 iB .. number of amplitude quantization bits
%                 Aoff .. amplitude quantizaion level offset
%                 Poff .. phase offset
%                 iDitPoly .. dither polynomial degree
%                 iDitBits .. number of dither bits used for
%                             dithering (<=iDitPoly)
%                 rcGen .. row of cells with primitive polynomial
%                          coefitients
%                 PsdFloor .. negative limit on psd in [dB]
%
%               DDS simulation input flags (0-no or 1-yes values):
%                 gShell .. running in GUI
%                 gPrintMsg .. print the execution progress
%                              to the MATLAB window
%                 gDit   .. dither the phase
%                 gTrunc .. truncate the phase accumulator
%                 gQuant .. quantize the sin amplitude
%                 gClear .. clear the intermediate results conserving
%                           the memory (if this one is set to 1 it is
%                           not possible to plot time domain plots)
%
%   Output:     rPsdQ .. power spectrum calculated from the
%                        DDS output signal rSigq
%               iQ    .. number of phase accumulator truncated
%                        bits
%               iLen  .. length (in samples) of one exact period
%                        of the time domain signal, which corresponds
%                        to the length of rPsdQ output spectrum vector.
%                        It may be in size of MBs if dithering is used!!
%
%               Time domain signals (if gClear == 0):
%                 rSigp .. phase
%                 rSigd .. dithered phase
%                 rSigt .. truncated (and dithered) phase
%                 rSigs .. non-quantized sin amplitude
%                 rSigq .. quantized amplitude
%
%   Calls:      ddsph.m
%               pn.m
%               vectbin.m
%               ddstrunc.m
%               ddssin.m
%               ddsqn.m
%               ddspsd.m
%
%   Author: Martin Pechanec, Febraury 14, 1998
%           pechanec@iname.com
% ----------------------------------------------------------------

% -- Check the shell presence and if not run from the shell
% then print the simulation messages
if 0 == gShell,
  gPrintMsg = 1;
end

iQ = iN - iP;  % Truncated bits

% -- Set the text message
iHandleRunMsg = findobj( gcbf, 'Tag', 'Text_Message' );
if 1 == gPrintMsg
  disp( '> ---------------- ' );
end

% -- Set parameters
sMessage = [ '> Parameter setting ...' ];
set( iHandleRunMsg, 'String', sMessage );
if 1 == gPrintMsg
  disp( sMessage );
end

% Read the values from the shell window
if 1 == gShell
  ddsgset;
end

% -- Generate one period of the phase
sMessage = ['> Phase generation ...'];
set( iHandleRunMsg, 'String', sMessage );
if 1 == gPrintMsg
  disp( sMessage );
end
rSigp = ddsph( iN, iF, 1 );

% -- Dither the phase .. LSBs
if 1 == gDit
  sMessage = [ '> Dither generation ... ' ];
  set( iHandleRunMsg, 'String', sMessage );
  if 1 == gPrintMsg
    disp( sMessage );
  end

% Set the dithering polynomial
  rGen = rcGen{ iDitPoly };

% Generate the dithering sequence
  mSeq = pn( rGen );
  cDither = vectbin( mSeq(:,1:iDitBits), 'high' );

% Calculate the one period length
  iLen = lcm( length( rSigp ), length( cDither ) );
  iL = fix( iLen/1024 );
  sLength = sprintf( ' (L=%.0fKs) ', iL );
  sMessage = ['> Phase dithering' sLength '...'];
  set( iHandleRunMsg, 'String', sMessage );
  if 1 == gPrintMsg
    disp( sMessage );
  end

% Generate both sequences of the length iLen
  iSigPer = iLen / length( rSigp );
  iDitherPer = iLen / length( cDither );

  mSig = rSigp' * ones( 1, iSigPer );
  rSigd = mSig(:)';

  mDither = cDither * ones( 1, iDitherPer );
  rDither = mDither(:)';

% -- Dither the signal
  rSigd = rSigd + rDither;
else
  rSigd = rSigp;
end

% -- Set the sequence length for the rest of the simulation
iL = length( rSigd );
if 15000 <= iL,
  iL = fix( iL/1024 );
  sLength = sprintf( ' (L=%.0fKs) ', iL );
else
  sLength = sprintf( ' (L=%.0fs) ', iL );
end

% Clear
if 1 == gClear
  clear rSigp;
end

% -- Truncate the phase
if 1 == gTrunc
  sMessage = ['> Phase truncation' sLength '...'];
  set( iHandleRunMsg, 'String', sMessage );
  if 1 == gPrintMsg
    disp( sMessage );
  end

  rSigt = ddstrunc( iQ, rSigd );
else
  rSigt = rSigd;
end

% Clear
if 1 == gClear
  clear rSigd;
end

% -- Phase --> amplitude conversion
sMessage = ['> Phase to amplitude conversion' sLength '...'];
set( iHandleRunMsg, 'String', sMessage );
if 1 == gPrintMsg
  disp( sMessage );
end
if 1 == gTrunc,
  rSigs = ddssin( iP, rSigt, Poff );
else
  rSigs = ddssin( iN, rSigt, Poff );
end

% Clear
if 1 == gClear
  clear rSigt;
end

% -- Amplitude quantization
if 1 == gQuant
  sMessage = ['> Amplitude quantization' sLength '...'];
  set( iHandleRunMsg, 'String', sMessage );
  if 1 == gPrintMsg
    disp( sMessage );
  end

  [rSigq, rE] = ddsqn( iB, rSigs, Aoff );
else
  rSigq = rSigs;
end

% Clear
if 1 == gClear
  clear rSigs;
end

% ---- Power spectrum calculation
sMessage = ['> Power spectrum calculation' sLength '...'];
set( iHandleRunMsg, 'String', sMessage );
if 1 == gPrintMsg
  disp( sMessage );
end

rPsdQ = ddspsd( rSigq, 'dB', PsdFloor );

sMessage = ['> Idle' sLength];
set( iHandleRunMsg, 'String', sMessage );
if 1 == gPrintMsg
  disp( sMessage );
end

% -- Power spectrum plotting commands
if 0 == gShell,
  disp( '> Spectrum: ddsfplot, Time: ddsftime' );
end
