function roPsd = ddspsd( riX, siOption, fiFloor )
%DDS power spectrum calculatoin and plotting
%   Y = DDSPSD(X,O) calculates the power spectrum of one the sequence
%       X. X has to be exactly one period of a signal (rectangular window)
%       O is the optional option string bellow.
%   Y = DDSPSD(X,O,F) same as above, but the noise floor is set to F [dB]
%
%   Power options:
%       'ck'      .. Four. coef. absolute value |c(k)|=1/N*|F(k)|
%       'ckpower' .. power  2*|c(k)|^2 = 2*( (1/N)*|F(k)| )^2
%       'dB'      .. power in [dB] relative to the theoretical 
%                    fundamental 20*log( 2*(1/N)*|F(k)| )
%       'dBrel'  .. power in [dB] relative to the maximum power
%
%   If no options are used then the 'dB' is assumed

%   Author: Martin Pechanec, October 27, 1997
%           pechanec@iname.com

% -- Check the input arguments. Set the floor for zeros in [dB] spectrum in [dB]
switch nargin
  case 1
    siOption = 'dB';
    fiFloor = -120;
    
  case 2
    fiFloor = -120;
    
  case 3
  otherwise
    error( 'DDSPSD: Wrong number of input parameters. It can be 1, 2, or 3!' );
end

% -- Check the input signal vector
[iR,iC] = size( riX );
if iC == 1,
  riX = riX.';   % Make it a row
end

% -- Calculate the DFT{riX}
iN = length( riX );
roPsd = abs( fft( riX ) );

% -- Adjust the output according to the options
switch siOption

% -- Amplitude |c(k)| plot
  case 'ck'
    roPsd = (1/iN) .* roPsd;
    
% -- Power 2*|c(k)|^2 at the frequency (valid only till the 1/2 f_s)  
  case 'ckpower'
    roPsd = 2 .* ( ((1/iN) .* roPsd).^2 );
    roPsd(1) = 1/2 * roPsd(1);	% Adjust the DC

% -- Relative [dB] power wrt theoretical fundamental 20*log( 2*(1/N)*|F(k)| )
  case 'dB'

%   Find the possible zeros and replace it with the fiFloor [dB] constant (10^(fiFloor/20)) 
    Min = 10^(fiFloor/20);
    roPsd = (2/iN) .* roPsd;
    roPsd(1) = 1/2 * roPsd(1);	% Adjust the DC
    roPsd( find( roPsd < Min ) ) = Min;
    roPsd = 20 .* log10( roPsd );

% -- Relative [dB] power wrt the actual fundamental power
  case 'dBrel'
    
%   Find the maximum
    roPsd = (2/iN) .* roPsd;
    roPsd(1) = 1/2 * roPsd(1);	% Adjust the DC
    MaxdB = 20 * log10( max( roPsd ) );
    
%   Find the possible zeros and replace it with the fiFloor [dB] constant (10^(fiFloor/20)) 
    Min = 10^((fiFloor+MaxdB)/20);
    roPsd( find( roPsd < Min ) ) = Min;
    roPsd = 20 .* log10( roPsd ) - MaxdB;
    
% -- Wrong input parameter  
  otherwise
    error( 'DDSPSD: Wrong option. Options: ck, ckpower, dB, dBrel.' );
end
