function [roY, roE] = ddsqn( iiB, riX, fiFrac )
%DDS quantization of the signal. The input must be in <-1,1> interval
%in the amplitude
%   Y = DDSQN(B,X) quantizes the X vector to B bits.
%   Y = DDSQN(B,X,F) scales the input vector by (1-fiFrac*A) where
%       A is the width of the quantization level; a = 2/2^b
%
%   [Y,E] = DDSQN( ... ) returns the quantized error in vector E,
%       E = Y - X = Quantized - Original
%
%   Quantization:
%
%       A ----------
%          -- Quantized value
%       0 --------------
%          -- Quantized value
%      -A ----------

%   Author: Martin Pechanec, October 22, 1997
%           pechanec@iname.com

% -- Check the input argument sizes
[iR,iC] = size( iiB );
if ( iC ~= 1 | iR ~= 1 ),
  error( 'DDSQN: Input B cannot be a vector, integer is expected.' );
end;

% Convert the input vector to the row vector
[iR,iC] = size( riX );
if iC == 1,
  riX = riX.';   % Make it a row
end

% Set the width of a quantization level. iiB levels from <-1;1>
A = 2/(2^iiB);

% Scale the input 
if ( nargin >= 3 )
  riX = (1 - fiFrac * A) .* riX;
end

% -- Quantize the vector
roY = A .* floor( riX ./ A ) + A/2;
roY( find( roY <= -1+A/2 ) ) = -1 + A/2;
roY( find( roY >= 1-A/2 ) ) = 1 - A/2;

% Calculate the quantization error
if nargout > 1,
  roE = roY - riX;
end